package net.knarcraft.blockhunt.arena;

import org.bukkit.Location;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * The locations of an arena
 */
public class ArenaLocations {

    private final @NotNull Location corner1;
    private final @NotNull Location corner2;
    private Location lobbyWarp;
    private Location hidersWarp;
    private Location seekersWarp;
    private Location spawnWarp;

    /**
     * Instantiates a new arena locations container
     *
     * @param corner1     <p>First corner of this arena's region</p>
     * @param corner2     <p>Second corner of this arena's region</p>
     * @param lobbyWarp   <p>The location of the lobby's warp</p>
     * @param hidersWarp  <p>The location of the hiders' warp</p>
     * @param seekersWarp <p>The location of the temporary seekers warp (should not be able to see hiders from this location)</p>
     * @param spawnWarp   <p>The location of the spawn warp where players are sent once this arena's game ends</p>
     */
    public ArenaLocations(@NotNull Location corner1, @NotNull Location corner2, @Nullable Location lobbyWarp,
                          @Nullable Location hidersWarp, @Nullable Location seekersWarp, @Nullable Location spawnWarp) {
        this.corner1 = corner1;
        this.corner2 = corner2;
        this.lobbyWarp = lobbyWarp;
        this.hidersWarp = hidersWarp;
        this.seekersWarp = seekersWarp;
        this.spawnWarp = spawnWarp;
    }

    /**
     * Gets the first corner of this arena's region
     *
     * @return <p>The first corner of this arena's region</p>
     */
    public @NotNull Location getCorner1() {
        return corner1;
    }

    /**
     * Gets the second corner of this arena's region
     *
     * @return <p>The second corner of this arena's region</p>
     */
    public @NotNull Location getCorner2() {
        return corner2;
    }

    /**
     * Gets the location of the lobby's warp
     *
     * @return <p>The location of the lobby's warp</p>
     */
    public @Nullable Location getLobbyWarp() {
        return lobbyWarp;
    }

    /**
     * Sets the location of the lobby's warp
     *
     * @param lobbyWarp <p>The new location of the lobby's warp</p>
     */
    public void setLobbyWarp(@NotNull Location lobbyWarp) {
        this.lobbyWarp = lobbyWarp;
    }

    /**
     * Gets the location of the hiders' warp
     *
     * @return <p>The location of the hiders' warp</p>
     */
    public @Nullable Location getHidersWarp() {
        return hidersWarp;
    }

    /**
     * Sets the location of the hiders' warp
     *
     * @param hidersWarp <p>The new location of the hiders' warp</p>
     */
    public void setHidersWarp(@NotNull Location hidersWarp) {
        this.hidersWarp = hidersWarp;
    }

    /**
     * Gets the location of the seekers' initial warp
     *
     * <p>Seekers are teleported here while the hiders are given time to hide.</p>
     *
     * @return <p>The location of the seekers' initial warp</p>
     */
    public @Nullable Location getSeekersWarp() {
        return seekersWarp;
    }

    /**
     * Sets the location of the seekers' initial warp
     *
     * @param seekersWarp <p>The new location of the seekers' initial warp</p>
     */
    public void setSeekersWarp(@NotNull Location seekersWarp) {
        this.seekersWarp = seekersWarp;
    }

    /**
     * Gets the location of the "spawn" warp
     *
     * <p>This location is where players are teleported to when leaving the arena</p>
     *
     * @return <p>The location of the "spawn" warp</p>
     */
    public @Nullable Location getSpawnWarp() {
        return spawnWarp;
    }

    /**
     * Sets the location of the "spawn" warp
     *
     * @param spawnWarp <p>The location of the "spawn" warp</p>
     */
    public void setSpawnWarp(@NotNull Location spawnWarp) {
        this.spawnWarp = spawnWarp;
    }

}
