package net.knarcraft.blockhunt.listener;

import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.MemoryStorage;
import net.knarcraft.blockhunt.PlayerHandler;
import net.knarcraft.blockhunt.arena.Arena;
import net.knarcraft.blockhunt.arena.ArenaHandler;
import net.knarcraft.blockhunt.arena.ArenaLocations;
import net.knarcraft.blockhunt.arena.ArenaState;
import org.bukkit.Effect;
import org.bukkit.Location;
import org.bukkit.Sound;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerMoveEvent;

import java.util.function.BiFunction;

public class OnPlayerMoveEvent implements Listener {

    @EventHandler(priority = EventPriority.NORMAL, ignoreCancelled = true)
    public void onPlayerMoveEvent(PlayerMoveEvent event) {
        // Early exit if no one is in any arena
        if (ArenaHandler.noPlayersInArenas()) {
            return;
        }
        Player player = event.getPlayer();
        for (Arena arena : MemoryStorage.arenaMap.values()) {
            //TODO: Optimize this to runtime O(1)
            if (!arena.playersInArena.contains(player)) {
                continue;
            }

            if (arena.gameState != ArenaState.IN_GAME) {
                return;
            }

            ArenaLocations locations = arena.arenaLocations;
            MemoryStorage.moveLoc.put(player, player.getLocation());
            if (locations.getCorner1() == null || locations.getCorner2() == null) {
                BlockHunt.plugin.getLogger().info("Arena:" +
                        arena.arenaName + " appears to have bad coords : pos1:" +
                        ((locations.getCorner1() != null) ? locations.getCorner1().toString() : " NULL") + " Pos2:" +
                        ((locations.getCorner2() != null) ? locations.getCorner2().toString() : " NULL"));
                BlockHunt.plugin.getLogger().info("Player has been returned to hiderswarp due to bad arena state");
                //event.setCancelled(true);
                Location loc = player.getLocation();
                player.playEffect(loc, Effect.ENDER_SIGNAL, null);
                player.playSound(loc, Sound.ENTITY_GHAST_SHOOT, 1, 1);
                PlayerHandler.teleport(player, locations.getHidersWarp());
                return;
            }

            // Force the player back in bounds if they are about to leave the arena
            Location maxLocation = getRelativeLocation(locations.getCorner1(), locations.getCorner2(), Math::max);
            Location minLocation = getRelativeLocation(locations.getCorner1(), locations.getCorner2(), Math::min);
            Location playerLocation = player.getLocation();
            if (!areIntersecting(playerLocation, minLocation, maxLocation)) {
                player.playEffect(playerLocation, Effect.ENDER_SIGNAL, null);
                player.playSound(playerLocation, Sound.ENTITY_GHAST_SHOOT, 1, 1);
                PlayerHandler.teleport(player, locations.getHidersWarp());
            }
            return;
        }
    }

    /**
     * Gets a location relative to the two given locations
     *
     * @param location1 <p>The first location</p>
     * @param location2 <p>The second location</p>
     * @param function  <p>The function to apply on the two locations' X, Y, Z properties to get new X, Y, Z values</p>
     * @return <p>A location relative to the two input locations</p>
     */
    private Location getRelativeLocation(Location location1, Location location2,
                                         BiFunction<Double, Double, Double> function) {
        return new Location(location1.getWorld(), function.apply(location1.getX(), location2.getX()),
                function.apply(location1.getY(), location2.getY()), function.apply(location1.getZ(), location2.getZ()));
    }

    /**
     * Checks if the given location is intersecting with the hit-box defined by the given location pair
     *
     * @param point       <p>The point to check for intersection</p>
     * @param locationMin <p>The minimum corner of the hit-box to check against</p>
     * @param locationMax <p>The maximum corner of the hit-box to check against</p>
     * @return <p>True if the point is intersecting with the hit-box</p>
     */
    private static boolean areIntersecting(Location point, Location locationMin, Location locationMax) {
        return (point.getBlockX() >= locationMin.getBlockX() && point.getBlockX() <= locationMax.getBlockX()) &&
                (point.getBlockY() >= locationMin.getBlockY() && point.getBlockY() <= locationMax.getBlockY()) &&
                (point.getBlockZ() >= locationMin.getBlockZ() && point.getBlockZ() <= locationMax.getBlockZ());
    }

}
